@@ -4,10 +4,13 @@ from aiohttp_socks import ProxyType, ProxyConnector, ChainProxyConnector, ProxyE
 from mo_dots import to_data, Data, DataObject, Null, NullType, FlatList, LIST
 from random import randrange
 import requests
-from typing import Union, Any
+from typing import Dict, List, Literal, Tuple, Union, Any
 
 proxies, one_hot = [], []
 has_init = False
+ProxyProto = Literal['socks4', 'socks5', 'http', 'https']
+DumpFormat = Literal['json', 'csv']
+Boolean = Literal[True, False]
 
 
 class ActError(Exception):
@@ -19,7 +22,7 @@ class ActError(Exception):
 		super().__init__(self.message)
 
 
-def act_parse_json(proxy_items: list) -> Union[list, None]:
+def act_parse_proxies(proxy_items: List[str]) -> Union[List[Dict], None]:
 	"""
 	Parse the proxy list JSON supplied by ActProxy API.
 		:rtype: Union[list, None]
@@ -41,7 +44,7 @@ def act_parse_json(proxy_items: list) -> Union[list, None]:
 	return _proxies
 
 
-def init(api_keys: list, output_format='json', get_userpass=True) -> Union[list, None]:
+def init(api_keys: List[str], output_format: DumpFormat = 'json', get_userpass: Any = True) -> Union[FlatList, str, None]:
 	"""
 	Synchronously initialize ActProxy API & return proxies from account.
 		:rtype: Union[list, None]
@@ -56,7 +59,7 @@ def init(api_keys: list, output_format='json', get_userpass=True) -> Union[list,
 		raise ValueError('api_keys must be a list containing at least one API key')
 	userpass = 'true' if get_userpass else 'false'
 	formatter = '&format=json' if output_format == 'json' else ''
-	proxies = []
+	proxies, proxy_items = [], []
 	proxies_csv = ''
 	for key_num, api_key in enumerate(api_keys):
 		api_url = f'https://actproxy.com/proxy-api/{api_key}?userpass={userpass}{formatter}'
@@ -67,8 +70,8 @@ def init(api_keys: list, output_format='json', get_userpass=True) -> Union[list,
 			else:
 				proxy_items = resp.text.splitlines()
 				proxies_csv += resp.text
-			_proxies = act_parse_json(proxy_items)
-			if isinstance(_proxies, list) and len(_proxies):
+			_proxies = act_parse_proxies(proxy_items)
+			if isinstance(_proxies, list) and len(_proxies) > 0:
 				proxies.extend(_proxies)
 				has_init = True
 			else:
@@ -79,10 +82,10 @@ def init(api_keys: list, output_format='json', get_userpass=True) -> Union[list,
 		one_hot = [0 for p in proxies]
 		return to_data(proxies) or None
 	else:
-		return proxies_csv or None
+		return proxies_csv if len(proxy_items) > 0 else None
 
 
-async def aioinit(api_keys: list = None, output_format='json', get_userpass=True) -> Data:
+async def aioinit(api_keys: List = None, output_format: DumpFormat = 'json', get_userpass: Boolean = True) -> Union[FlatList, str, None]:
 	"""
 	Asynchronously initialize ActProxy API & return proxies from account.
 		:param api_keys: List of ActProxy.com API keys.
@@ -98,7 +101,7 @@ async def aioinit(api_keys: list = None, output_format='json', get_userpass=True
 	async with ClientSession() as session:
 		userpass = 'true' if get_userpass else 'false'
 		formatter = '&format=json' if output_format == 'json' else ''
-		proxies = []
+		proxies, proxy_items = [], []
 		proxies_csv = ''
 		for key_num, api_key in enumerate(api_keys):
 			api_url = f'https://actproxy.com/proxy-api/{api_key}?userpass={userpass}{formatter}'
@@ -106,25 +109,27 @@ async def aioinit(api_keys: list = None, output_format='json', get_userpass=True
 				if resp.status == 200:
 					if output_format == 'json':
 						proxy_items = await resp.json(content_type=None)
-						_proxies = act_parse_json(proxy_items)
-						if isinstance(_proxies, list) and len(_proxies):
-							proxies.extend(_proxies)
-							has_init = True
-						else:
-							raise TypeError(f"ActProxy API Key #{key_num} didn't return a proxy list")
+
 					else:
-						proxies_csv += await resp.text()
+						resp_body = await resp.text()
+						proxy_items = resp_body.splitlines()
+						proxies_csv += resp_body
+					_proxies = act_parse_proxies(proxy_items)
+					if isinstance(_proxies, list) and len(_proxies):
+						proxies.extend(_proxies)
+						has_init = True
+					else:
+						raise TypeError(f"ActProxy API Key #{key_num} didn't return a proxy list")
 				else:
 					raise ActError(f"HTTP error {resp.status} contacting ActProxy.")
 		if output_format == 'json':
 			one_hot = [0 for p in proxies]
 			return to_data(proxies) or None
 		else:
-			proxy_lines = proxies_csv.splitlines()
-			return to_data(proxy_lines) if len(proxy_lines) else None
+			return proxies_csv if len(proxy_items) > 0 else None
 
 
-def rotate(protocol='socks5'):
+def rotate(protocol: ProxyProto = 'socks5') -> Data:
 	"""
 	Get the next proxy in the one-hot rotation for use with the requests[socks] package after having once run
 	actproxy.init(output_format='json').
@@ -139,7 +144,8 @@ def rotate(protocol='socks5'):
 	return to_data(requests_proxy)
 
 
-def aiohttp_rotate(protocol: str = 'socks5', return_proxy: Any = False) -> Union[ProxyConnector, tuple]:
+def aiohttp_rotate(protocol: ProxyProto = 'socks5',
+                   return_proxy: Boolean = False) -> Union[ProxyConnector, Tuple[Data, ProxyConnector]]:
 	"""
 	Get an aiohttp connector that uses the next proxy in the one-hot rotation after having once run
 	actproxy.aioinit(output_format='json').
@@ -151,10 +157,13 @@ def aiohttp_rotate(protocol: str = 'socks5', return_proxy: Any = False) -> Union
 	proxy_connector = ProxyConnector.from_url(
 		f'{protocol}://{proxy.username}:{proxy.password}@{proxy.host}:{proxy.port}'
 	)
-	return proxy, proxy_connector if return_proxy else proxy_connector
+	if return_proxy:
+		return proxy, proxy_connector
+	else:
+		return proxy_connector
 
 
-def random_proxy(protocol: str = 'socks5') -> Data:
+def random_proxy(protocol: ProxyProto = 'socks5') -> Data:
 	"""
 	Get a random proxy from your account after having once run actproxy.init(output_format='json') or
 	actproxy.aioinit(output_format='json')
@@ -171,7 +180,8 @@ def random_proxy(protocol: str = 'socks5') -> Data:
 	})
 
 
-def aiohttp_random(protocol='socks5', return_proxy: Any = False) -> Union[ProxyConnector, tuple]:
+def aiohttp_random(protocol: ProxyProto = 'socks5',
+                   return_proxy: Boolean = False) -> Union[ProxyConnector, Tuple[Data, ProxyConnector]]:
 	"""
 	Get a random aiohttp connector after having once run actproxy.aioinit(output_format='json')
 		:param protocol: 'socks5' or 'http'; must be the same as your ActProxy proxies.
@@ -181,10 +191,13 @@ def aiohttp_random(protocol='socks5', return_proxy: Any = False) -> Union[ProxyC
 	proxy = random_proxy(protocol)
 	proxy = proxy.https
 	proxy_connector = ProxyConnector.from_url(proxy)
-	return proxy, proxy_connector if return_proxy else proxy_connector
+	if return_proxy:
+		return proxy, proxy_connector
+	else:
+		return proxy_connector
 
 
-def one_hot_proxy():
+def one_hot_proxy() -> Data:
 	"""
 	Get the next proxy in the one-hot rotation and flip the next proxy bit hot.
 		:return: mo-dots (Dict-like) object containing the proxy's various parameters.
